""" ZSS Rule """
__version__='3.1'

from Globals import HTMLFile, MessageDialog
from OFS import SimpleItem 
from OFS import ObjectManager
from OFS import PropertyManager

import re
import string

import BrowserType
import ZSSAttribute

def translateSelector(selector):
    """ Translate a selector into an ID """
    id=string.lower(re.sub("( +|\.|#|:)", "_", string.join(selector)))
    if id[0]=="_":
        id = "z" + id
    else:
        id=string.upper(id[0]) + id[1:]
    return id

def manage_addZSSRuleForm(self, REQUEST):
    """ Add a ZSSRule object """
    strret = "<html><head><title>Add Z Style Sheet Rule</title></head>"
    strret = strret + '<body><h1 align=center>Add Z Style Sheet Rule</h1><form action="manage_addZSSRule" method="POST">'
    strret = strret + '<div align=center><center><table cellspacing="2">'
    strret = strret + '<tr><th align="LEFT" valign="TOP">Id</th><td align="LEFT" valign="TOP"><input type="TEXT" name="id" size="20"></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP"><em>Title</em></th><td align="LEFT" valign="TOP"><input type="TEXT" name="title" size="20"></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP">selector</th><td align="LEFT" valign="TOP"><textarea name="selector:lines" rows="6" cols="35"></textarea></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP">Browsers</th><td align="LEFT" valign="TOP"><select name="Browsers" size="3" multiple>'
    for i in BrowserType.ListBrowsers():
        strret = strret + '<option selected name="%s">%s</option>' % (i, i)
    strret = strret + '</select></td></tr><tr><td></td><td>'
    strret = strret + '<input type="SUBMIT" value="Add"></td></tr>'
    strret = strret + '</table></center></div></form></body></html>'
    return strret

def manage_addZSSRule(self, id, title='', selector=[], Browsers=[], REQUEST=None):
    """ Add a ZSSRule object """
    obj=ZSSRule(id, title, selector, Browsers, REQUEST)
    self._setObject(obj.id, obj)
    if REQUEST is not None: 
        return self.manage_main(self, REQUEST)

manageAddForm = manage_addZSSRuleForm
manageAdd = manage_addZSSRule

class ZSSRule(	ObjectManager.ObjectManager,
                PropertyManager.PropertyManager,
                SimpleItem.SimpleItem,
                BrowserType.BrowserType):
    """ Representation of Cascading Style Sheet Rules.
        Parent of ZSSRule, ZSSSelector and ZSSAttribute objects. """
    manage_addZSSRuleForm=manageAddForm
    manage_addZSSRule=manageAdd
    manage_addZSSAttrForm=ZSSAttribute.manage_addZSSAttrForm
    manage_addZSSAttr=ZSSAttribute.manage_addZSSAttr
    manage=manage_main=manage_workspace=HTMLFile('manage_main', globals())
    manage_propertiesForm=HTMLFile('properties', globals())
    manage_ChangeBrowserForm=HTMLFile('manage_ChangeBrowser', globals())

    meta_type    = 'ZSS Rule'
    id        = 'ZSSRule'
    icon        = 'misc_/ZStyleSheet/ZSSRule'
    meta_types    = (
            {'name': 'ZSS Rule', 'action': 'manage_addZSSRuleForm'},
            {'name': 'ZSS Attribute', 'action': 'manage_addZSSAttrForm'},
            )
    manage_options    = (
            {'label': 'Contents', 'action' : 'manage_main'},
            {'label': 'Properties', 'action': 'manage_propertiesForm'},
            {'label': 'Preview', 'action': 'preview'},
            {'label': 'Change Browser', 'action': 'manage_ChangeBrowserForm'},
            {'label': 'Import/Export', 'action': 'manage_importExportForm'},
            {'label': 'Undo', 'action': 'manage_UndoForm'},
            )
    __ac_permissions__= (
            ('Manage properties',
                ('manage_addProperty',
                'manage_editProperties',
                'manage_delProperties',
                'manage_changeProperties',
                )
            ),
            ('Access contents information',
                ('hasProperty',
                'propertyIds',
                'propertyValues',
                'propertyItems',
                '',
                ),
                ('Anonymous', 'Manager'),
                        ),
            )
    _properties=(
        {'id' : 'title', 'type': 'string', 'mode': 'w'},
        {'id' : 'selector', 'type': 'lines', 'mode': 'w'},
        {'id' : 'Browsers', 'type' : 'multiple selection', 'mode': 'w', 'select_variable': 'BrowserList'},
        )

    def __init__(self, id, title='', selector=[], Browsers=[], REQUEST=None):
        """ Initialise a ZSSRule """
        if id:
            self.id=id
        else:
            self.id=translateSelector(selector)
        self.title=title
        self.selector=selector
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers

    def manage_ChangeBrowser(self, Browsers=[], REQUEST=[]):
        """ Walk the tree, changing the Browsers entry at each node """
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers
        objs=self.objectValues('ZSS Rule')
        for i in objs:
            i.manage_ChangeBrowser(Browsers)
        objs=self.objectValues('ZSS Attribute')
        for i in objs:
            i.Browsers=Browsers
        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='The browser settings have been changed',
                action ='manage_main')

    def calccontext(self, parentcontext=[], REQUEST=None):
        """ Calculate the context relative to the passed parent context(s) """
        context=[]
        if parentcontext:
            for i in parentcontext:
                for s in self.selector:
                    context.append(i+" "+s)
        else:
            for s in self.selector:
                context.append(s)
        return context

    def dump(self, parentcontext=[], REQUEST=None):
        """ Dump this rule, showing the browser restrictions
            in comments. """
        context=self.calccontext(parentcontext, REQUEST)
        strret=""
        for i in context:
            if strret != "":
                strret = strret + ", "
            strret = strret + "%s" % i
        strret = "/* Browsers=%s */\n%s" % (self.Browsers, strret)
        strret = strret + " {\n"
        objs=self.objectValues('ZSS Attribute')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "%s\n" % i.dump(REQUEST)
        strret=strret + "}\n"
        objs=self.objectValues('ZSS Rule')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "%s\n" % i.dump(context, REQUEST)
        return strret

    def children(self, parentcontext=[], REQUEST=None):
        """ Display the stylesheet from here down """
        b=""
        for i in self.Browsers:
            if (self.isBrowser(i, REQUEST)!=0):
                b=i
        if (b==""):
            return "/* %s - Excluded for this browser */" % self.id
        context=self.calccontext(parentcontext, REQUEST)
        strret=""
        for i in context:
            if strret != "":
                strret = strret + ", "
            strret = strret + "%s" % i
        strret = strret + " {\n"
        objs=self.objectValues('ZSS Attribute')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "\t%s\n" % i.__str__(REQUEST)
        strret=strret + "}\n"
        objs=self.objectValues('ZSS Rule')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "%s\n" % i.__str__(context, REQUEST)
        return strret

    def index_html(self, REQUEST):
        """ Render this rule ONLY as a style= attribute """
        b=""
        for i in self.Browsers:
            if (self.isBrowser(i, REQUEST)):
                b=i
        if (b==""):
            return ""
        strret='style="'
        objs=self.objectValues('ZSS Attribute')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + i.__str__(REQUEST)
        strret=strret + '"'
        return strret

    def preview(self, REQUEST):
        """ Render a human viewable version of this rule """
        strret = "<html><head><title>Preview of ZStyleSheet Rule %s</title></head>" % self.title_and_id()
        strret = strret + self.style(REQUEST)
        strret = strret + "<body><h1 align=center>Zope Style Sheet Rule Preview</h1>"
        strret = strret + "<h2 align=center>%s</h2>" % self.title_and_id()
        strret = strret + "<blockquote><pre>%s</pre></blockquote>" % self.children([], REQUEST)
        strret = strret + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        strret = strret + "<body></html>"
        return strret

    def __str__(self, parentcontext=[], REQUEST=None):
        """ Render this rule """
        return "/* ZSS Rule: %s */\n%s" % (self.title_and_id(), self.children(parentcontext, REQUEST))

    def tag(self, REQUEST):
        """ """
        return self.__str__(REQUEST)

    def style(self, REQUEST):
        """ Render the rule as a <style> </style> section """
        return '<style type="text/css"><!--\n%s\n--></style>' % (self.children([], REQUEST))
