""" Zope Style Sheets """
__version__='3.1'

from Globals import HTML, HTMLFile, MessageDialog
from OFS import SimpleItem 
from OFS import ObjectManager
from OFS import PropertyManager
from AccessControl.Role import RoleManager
import string
import re

import BrowserType
import ZSSRule
import ZSSAttribute

StringType=type('')
 
def manage_addZStyleSheetForm(self, REQUEST):
    """ Add a ZStyleSheet object """
    lb=BrowserType.ListBrowsers()
    strret = "<html><head><title>Add Z Style Sheet </title></head>"
    strret = strret + '<body><h1 align=center>Add Z Style Sheet</h1>'
    strret = strret + '<form action="manage_addZStyleSheet" method="POST" enctype="multipart/form-data">'
    strret = strret + '<div align=center><center><table cellspacing="2">'
    strret = strret + '<tr><th align="LEFT" valign="TOP">Id</th><td align="LEFT" valign="TOP"><input type="TEXT" name="id" size="20"></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP"><em>Title</em></th><td align="LEFT" valign="TOP"><input type="TEXT" name="title" size="20"></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP"><em>Import File</em></th><td align="LEFT" valign="TOP"><input type="FILE" name="file" size="20"></td></tr>'
    strret = strret + '<tr><th align="LEFT" valign="TOP">Browsers</th><td align="LEFT" valign="TOP"><select name="Browsers" multiple size="%d">' % len(lb)
    for i in lb:
        strret = strret + '<option selected name="%s">%s</option>' % (i, i)
    strret = strret + '</select></td></tr>'
    strret = strret + '<tr><td></td><td><input type="SUBMIT" value="Add"></td></tr>'
    strret = strret + '</table></center></div></form></body></html>'
    return strret

def cookId(id, title, file):
    if not id and hasattr(file,'filename'):
        filename=file.filename
        title=title or filename
        id=filename[max(string.rfind(filename, '/'),
                        string.rfind(filename, '\\'),
                        string.rfind(filename, ':'),
                        )+1:]                  
    return id, title

def manage_addZStyleSheet(self, id, title='', file='', Browsers=[], REQUEST=None):
    """Add a ZStyleSheet object"""
    (id, title)=cookId(id, title, file)
    obj=ZStyleSheet(id, title, Browsers, REQUEST)
    self._setObject(id, obj)
    if (file):
        obj.manage_importCSS(file=file, Browsers=Browsers, REQUEST=REQUEST)
    if REQUEST is not None: 
        return self.manage_main(self, REQUEST)

def cmp_list(List1, List2):
    """ Compare the contents of two lists """
    List1.sort()
    List2.sort()
    if len(List1)>len(List2):
        return -1
    if len(List1)<len(List2):
        return 1
    cmp_list_loop=0
    while cmp_list_loop<len(List1):
        v=cmp(List1[cmp_list_loop], List2[cmp_list_loop])
        if v!=0:
            return v
        cmp_list_loop=cmp_list_loop+1
    return 0

def cmp_dict(Dict1, Dict2):
    """ Compare the contents of two dictionaries """
    List1=Dict1.keys()
    List2=Dict2.keys()
    List1.sort()
    List2.sort()
    if len(List1)>len(List2):
        return -1
    if len(List1)<len(List2):
        return 1
    cmp_dict_loop=0
    while cmp_dict_loop<len(List1):
        v=cmp(List1[cmp_dict_loop], List2[cmp_dict_loop])
        if v!=0:
            return v
        v=cmp(Dict1[List1[cmp_dict_loop]], Dict2[List2[cmp_dict_loop]])
        if v!=0:
            return v
        cmp_dict_loop=cmp_dict_loop+1
    return 0

class _Rule:
    """ """
    def __init__(self, Selectors, Attribs, List):
        """ """
        if type(Selectors)==type(''):
            Selectors=string.strip(Selectors)
            if string.find(Selectors, " ")!=0:
                Selectors=re.split(" ", Selectors)
        Attributes={}
        if type(Attribs)==type(''):
            for Attrib in re.split(";", Attribs):
                try:
                    (Id, Value)=re.split(":", Attrib)
                except:
                    pass
                else:
                    Id=string.strip(Id)
                    Value=string.strip(Value)
                    if Id:
                        Attributes[Id]=Value
        else:
            Attributes=Attribs
        self.Selector=[Selectors[0]]
        self.Attributes={}
        self.List=[]
        if len(Selectors)>1:
            self.List.append(_Rule(Selectors[1:], Attributes, self.List))
        else:
            self.Attributes=Attributes
        self._merge(List)

# This method creates the ZSSRule and ZSSAttribute objects and adds them to the Zope Tree
    def _create(self, Parent, Browsers):
        """ Create the objects listed in this node, and its children """
        if not hasattr(Parent, ZSSRule.translateSelector(self.Selector)):
            NewObj=ZSSRule.ZSSRule('', '', selector=self.Selector, Browsers=Browsers)
            Parent._setObject(NewObj.id, NewObj)
        else:
            NewObj=getattr(Parent, ZSSRule.translateSelector(self.Selector))
        for S in self.List:
            S._create(NewObj, Browsers)
        for K in self.Attributes.keys():
            if not hasattr(NewObj, K):
                NewObj._setObject(K, ZSSAttribute.ZSSAttr(K, self.Attributes[K], Browsers))

# This method allows you to dump the Rule Tree from here
    def _dump(self, Context='', StopAt=-1):
        """ """
        strret = "%s%s : %s\n" % (Context, self.Selector, self.Attributes)
        if StopAt!=0:
            for S in self.List:
                strret = strret + S._dump(Context + "%s->" % self.Selector, StopAt-1)
        return strret

    def _merge(self, List):
        """ """
        if type(List)!=type([]):
            raise "Type Error, List should be of type 'list' not %s" % re.sub("<type|>", "", "%s" % type(List))
        _merge_loop_1=0
        while _merge_loop_1<len(List):
            if self!=List[_merge_loop_1]:
                if cmp_list(self.List, List[_merge_loop_1].List)==0 and cmp_dict(self.Attributes, List[_merge_loop_1].Attributes)==0:
                    for S in List[_merge_loop_1].Selector:
                        self.Selector.append(S)
                    del List[_merge_loop_1]
                    _merge_loop_1=_merge_loop_1-1
            _merge_loop_1=_merge_loop_1+1
        _merge_loop_1=0
        while _merge_loop_1<len(List):
            if self!=List[_merge_loop_1]:
                if cmp_list(self.Selector, List[_merge_loop_1].Selector)==0:
                    for S in List[_merge_loop_1].List:
                        self.List.append(S)
                    for A in List[_merge_loop_1].Attributes.keys():
                        self.Attributes[A]=List[_merge_loop_1].Attributes[A]
                    del List[_merge_loop_1]
                    _merge_loop_1=_merge_loop_1-1
                    _merge_loop_2=0
                    while _merge_loop_2<len(List):
                        List[_merge_loop_2]._merge(List)
                        _merge_loop_2=_merge_loop_2+1
            _merge_loop_1=_merge_loop_1+1
        _merge_loop_1=0
        while _merge_loop_1<len(self.List):
            self.List[_merge_loop_1]._merge(self.List)
            _merge_loop_1=_merge_loop_1+1

class ZStyleSheet(  ObjectManager.ObjectManager,
                    PropertyManager.PropertyManager,
                    RoleManager,
                    SimpleItem.SimpleItem,
                    BrowserType.BrowserType):
    """ Representation of Cascading Style Sheets. """
    manage_addZSSRuleForm=ZSSRule.manage_addZSSRuleForm
    manage_addZSSRule=ZSSRule.manage_addZSSRule
    manage=manage_main=manage_workspace=HTMLFile('manage_main', globals())
    manage_propertiesForm=HTMLFile('properties', globals())
    manage_importCSSForm=HTMLFile('manage_importCSS', globals())
    manage_ChangeBrowserForm=HTMLFile('manage_ChangeBrowser', globals())

    meta_type       = 'Z Style Sheet'
    id              = 'ZStyleSheet'
    icon            = 'misc_/ZStyleSheet/ZStyleSheet'
    meta_types      = (
            {'name': 'ZSS Rule', 'action': 'manage_addZSSRuleForm'},
                    )
    manage_options  = (
            {'label': 'Contents', 'action' : 'manage_main'},
            {'label': 'Properties', 'action': 'manage_propertiesForm'},
            {'label': 'Preview', 'action': 'preview'},
            {'label': 'Change Browser', 'action': 'manage_ChangeBrowserForm'},
            {'label': 'Import CSS', 'action': 'manage_importCSSForm'},
            {'label': 'Import/Export', 'action': 'manage_importExportForm'},
            {'label': 'Undo', 'action': 'manage_UndoForm'},
            {'label': 'Security', 'action' : 'manage_access'},
                        )
    __ac_permissions__= (
            ('Manage properties',
                ('manage_addProperty',
                'manage_editProperties',
                'manage_delProperties',
                'manage_changeProperties',
                )
            ),
            ('Access contents information',
                ('hasProperty',
                'propertyIds',
                'propertyValues',
                'propertyItems',
                '',
                ),
                ('Anonymous', 'Manager'),
            ),
                    )
    _properties	=   (
            {'id': 'title', 'type': 'string', 'mode': 'w'},
            {'id': 'Browsers', 'type': 'multiple selection', 'mode': 'w', 'select_variable': 'BrowserList'},
                    )

    def __init__(self, id, title='', Browsers=[], REQUEST=None):
        """ Initialise a ZStyleSheet """
        self.id=id
        self.title=title
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers

    def manage_importCSS(self, file='', Browsers=[], REQUEST=[]):
        """ Load, interpret and import a CSS file """
        n=1<<16
        if type(file) is StringType:
            size=len(file)
            if size < n:
                data=file
            else:
                data=Pdata(file)
        else:
            seek=file.seek
            read=file.read
            seek(0,2)
            size=end=file.tell()
            if size <= 2*n:
                seek(0)
                if size < n:
                    data=read(size)
                else:
                    data=Pdata(read(size))

        if (not(Browsers)):
            Browsers=self.BrowserList()

        if not(data):
            return MessageDialog(
                title  ='Failed!',
                message='Your static stylesheet was empty!!',
                action ='manage_main')
        else:
            self._WalkCSS(data, Browsers)

        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='Your static stylesheet has been added',
                action ='manage_main')

    def manage_ChangeBrowser(self, Browsers=[], REQUEST=[]):
        """ Walk the tree, changing the Browsers entry at each node """
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers
        objs=self.objectValues('ZSS Rule')
        for i in objs:
            i.manage_ChangeBrowser(Browsers)
        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='The browser settings have been changed',
                action ='manage_main')

    def _WalkCSS(self, CSS, Browsers):
        """ Walk a CSS file, ignoring comments, extracting rules, selectors and attributes """
        CSS=re.sub("\n|\r|\t", " ", CSS)
        CSS=re.sub("/\*.*?\*/", " ", CSS)
        CSS=re.sub("}", "\n", CSS)
        for Rule in re.split("\n", CSS):
            try:
                (Selectors, Attributes)=re.split("{", Rule)
            except:
                pass
            else:
# Split the rules, merge and munge
                SelList=[]
                for Selector in re.split(",", Selectors):
                    SelList.append(_Rule(Selector, Attributes, SelList))
# Create the tree
                for S in SelList:
                    S._create(self, Browsers)

    def dump(self, REQUEST):
        """ Dump this sheet, showing the browser restrictions
            in comments. """
        strret = "/* Browsers=%s */\n" % (self.Browsers)
        objs=self.objectValues('ZSS Rule')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "%s\n" % i.dump([], REQUEST)
        strret=strret + "/* End of sheet */\n"
        return strret

    def children(self, REQUEST):
        """ Render the stylesheet from here down """
        b=""
        for i in self.Browsers:
            if (self.isBrowser(i, REQUEST)!=0):
                b=i
        if (b==""):
            return "/* %s - Excluded for this browser */" % self.id
        strret=""
        objs=self.objectValues('ZSS Rule')
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            strret = strret + "%s\n" % i.__str__([], REQUEST)
        strret=strret + "/* End of sheet */\n"
        return strret

    def index_html(self, REQUEST):
        """ Render the stylesheet as text/css """
        strret = "/* Cascading Style Sheet created using ZStyleSheet */\n"
        strret = strret + self.children(REQUEST)
        strret = strret + "/* Written by Haqa http://www.zope.org/Members/haqa */\n"
        REQUEST['RESPONSE'].setHeader('Content-Type', 'text/css')
        return strret

    def preview(self, REQUEST):
        """ Render a human viewable version of this stylesheet """
        strret = "<html><head><title>Preview of ZStyleSheet %s</title></head>" % self.title_and_id()
        strret = strret + self.style(REQUEST)
        strret = strret + "<body><h1 align=center>Zope Style Sheet Preview</h1>"
        strret = strret + "<h2 align=center>%s</h2>" % self.title_and_id()
        strret = strret + "<blockquote><pre>%s</pre></blockquote>" % self.children(REQUEST)
        strret = strret + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        strret = strret + "<body></html>"
        return strret

    def __str__(self):
        """ Add a LINK tag to the HTML page """
        return '<link rel="stylesheet" type="text/css" href="%s">' % (self.absolute_url())

    def tag(self):
        """ """
        return self.__str__()

    def style(self, REQUEST):
        """ Render the stylesheet as a <style> </style> section """
        return '<style type="text/css"><!--\n%s\n--></style>' % (self.children(REQUEST))
